/***************************************************************************
                          chemgraphitm.cpp  -  description
                             -------------------
    begin                : Fri Jan 25 2002
    copyright            : (C) 2002 by Ricardo Stefani - The chemsuite Project
    email                : rstefani@seul.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "chemgraphitm.h"
#include "chemdefs.h"
#include <qpainter.h>
#include <qcanvas.h>
#include <qimage.h>
#include <qpen.h>

#define abs(x) ((x > 0) ? x:-x)

chemGraphItem::chemGraphItem(QCanvas* canvas) :
  QCanvasRectangle(canvas) 
{
 setSize(60, 60);
 setType(_GR_AROMATIC);

  start_x = 0;
  start_y = 0;
  end_x = 0;
  end_y = 0;

}

chemGraphItem::~chemGraphItem()
{
hide();	
}


void chemGraphItem::ProcessMol()
{
}


void chemGraphItem::drawBond(QPainter& p,  BondStyle style)
{
setSize(30, 15);
if(style==SINGLE_BOND)
  {
    if(start_x==0)
   {
   QPointArray px(2);
    px[0] = QPoint(0,  14);
    px[1] = QPoint(29,  0);
    px.translate((int)x(), (int) y());
    pa = px;
    p.drawLine(px[0],  px[1]);
    } else
    {
    QPointArray px(2);
    setSize((int) abs(end_x),  (int) abs(end_y));
    px[0] = QPoint(0,  start_y);
    px[1] = QPoint(0,  end_y);
    px.translate((int)x(), (int) y());
    pa = px;
    p.drawLine(px[0],  px[1]);
    }
  }
}


void chemGraphItem::drawLabel(QPainter& p, QString text)
{

}


void chemGraphItem::drawCiclePentane(QPainter& p)
{
    QPointArray px(5);
    setSize(60, 45);

    px[0] = QPoint( 29, 0);
    px[1] = QPoint(59,  14);
    px[2] = QPoint(44,  44);
    px[3] = QPoint(14, 44);
    px[4] = QPoint(0, 14);


    px.translate((int)x(), (int) y());
    pa = px;

    p.drawPolygon(px);

}


void chemGraphItem::drawCiclePentadiene(QPainter& p)
{

   QPointArray px(5);
   QPointArray pl(2);

    setSize(60, 45);

    px[0] = QPoint( 29, 0);
    px[1] = QPoint(59,  14);
    px[2] = QPoint(44,  44);
    px[3] = QPoint(14, 44);
    px[4] = QPoint(0, 14);


    px.translate((int)x(), (int) y());
    pa = px;

    p.drawPolygon(px);

     pl[0] = QPoint(53,  16);
     pl[1] = QPoint(42,  41);
     pl.translate((int) x(), (int) y());
     p.drawLine(pl[0], pl[1]);

     pl[0] = QPoint(18,  41);
     pl[1] = QPoint(5, 16);
     pl.translate((int) x(), (int) y());
     p.drawLine(pl[0], pl[1]);


}


void chemGraphItem::drawCicleHexane(QPainter& p)
{
   QPointArray px(6);

    px[0] = QPoint( 29, 0);
    px[1] = QPoint(59,  14);
    px[2] = QPoint(59,  44);
    px[3] = QPoint(29,  59);
    px[4] = QPoint(0,  44);
    px[5] = QPoint(0, 14);

    px.translate((int)x(), (int) y());
    pa = px;

    p.drawPolygon(px);

}


void chemGraphItem::drawAromatic(QPainter& p)
{

    QPointArray px(6);
    QPointArray pl(2);

    px[0] = QPoint( 29, 0);
    px[1] = QPoint(59,  14);
    px[2] = QPoint(59,  44);
    px[3] = QPoint(29,  59);
    px[4] = QPoint(0,  44);
    px[5] = QPoint(0, 14);

    px.translate((int)x(), (int) y());
    pa = px;

    p.drawPolygon(px);

     pl[0] = QPoint(30,  4);
     pl[1] = QPoint(55, 16);
     pl.translate((int) x(), (int) y());
     p.drawLine(pl[0], pl[1]);

     pl[0] = QPoint(4,  43);
     pl[1] = QPoint(4,  17);
     pl.translate((int) x(), (int) y());
     p.drawLine(pl[0], pl[1]);

     pl[0] = QPoint(55, 43);
     pl[1] = QPoint(30, 55);
     pl.translate((int) x(), (int) y());
     p.drawLine(pl[0], pl[1]);

}


void chemGraphItem::drawShape(QPainter& p)
{
 setPen( QPen( QColor(0, 0, 0), 1,  SolidLine ) );

if(itmtype==_GR_CICLE_HEXANE)
    drawCicleHexane(p);
 if(itmtype==_GR_AROMATIC)
    drawAromatic(p);
 if(itmtype==_GR_CICLE_PENTANE)
    drawCiclePentane(p);
 if(itmtype==_GR_CICLE_PENTADIENE)
    drawCiclePentadiene(p);
 if(itmtype==_GR_SINGLE_BOND)
    drawBond(p, SINGLE_BOND);


// select the item
if(selected())
{
  setPen( QPen( QColor(128, 128, 128), 1,  DashLine ) );
  p.drawRect( rect() );
}



}

void chemGraphItem::moveBy(double dx, double dy)
{
    int idx = int(x()+dx)-int(x());
    int idy = int(y()+dy)-int(y());
    if ( idx || idy ) {
        hide();
        }
        QCanvasRectangle::moveBy(dx,dy);
        show();
        canvas()->update();
    
}


QPointArray chemGraphItem::points() const
{
 QPointArray pa1 = QCanvasRectangle::areaPoints();
 pa1.translate(int(x()),int(y()));
  return pa1;
}


void chemGraphItem::setStart(QPoint ps)
{
    start_x = ps.x();
    start_y  = ps.y();
}


void chemGraphItem::setEnd(QPoint ps)
{
  end_x = ps.x();
  end_y = ps.y();
}